#region "Copyright"
/********************************************************************************
 * Copyright (c) 2007, Kerry D. Wong
 * http://www.kerrywong.com
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *   * Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *   * Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *   * Neither the name of Kerry D. Wong nor the
 *     names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY KERRY D. WONG "AS IS" AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL KERRY D. WONG BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *******************************************************************************/
#endregion

#region "using"
using System;
using System.IO;
using System.Collections.Generic;
using System.Text;
#endregion

namespace KerryWong.TiffMS.Utils
{
    public class Config
    {
        #region "public consts"
        public const string ACTION_KEY = "ACTION";
        public const string ACTION_MERGE = "MERGE";
        public const string ACTION_SPLIT = "SPLIT";

        public const string SRC_DIR_KEY = "SOURCE_DIR";
        public const string DEST_DIR_KEY = "DESTINATION_DIR";
        public const string INPUT_FILE_KEY = "INPUT";
        public const string OUTPUT_FILE_KEY = "OUTPUT";
        #endregion

        #region "private consts"
        private const char COMMENT_CHAR = '#';
        private const string LINE_CONTINUATION = "...";
        #endregion

        #region "public functions"
        public ConfigurationValues GetConfigurations(string fileName, ref string err)
        {
            ConfigurationValues confValues = new ConfigurationValues();
            confValues.SrcDir = Environment.CurrentDirectory;
            confValues.DestDir = Environment.CurrentDirectory;
            confValues.OutputFile = string.Concat(Guid.NewGuid().ToString("N"), ".tiff");

            StreamReader sr = new StreamReader(fileName);

            string s = string.Empty;
            err = string.Empty;
            string lastLine = string.Empty;
            do
            {
                s = sr.ReadLine();
                if (s != null && s.Trim() != string.Empty)
                    ProcessLine(sr, s, ref confValues, ref err);
            } while (s != null);

            sr.Close();

            return confValues;
        }
        #endregion

        #region "private functions"
        private void ProcessLine(StreamReader sr, string s, ref ConfigurationValues confValues, ref string err)
        {
            string key = string.Empty;
            StringBuilder sb = new StringBuilder();

            try
            {
                if (s != string.Empty && s.Trim().IndexOf(COMMENT_CHAR) != 0)
                {
                    string str = s.Trim();
                    string r = string.Empty;

                    if (str.LastIndexOf(LINE_CONTINUATION) != str.Length - LINE_CONTINUATION.Length)
                    {
                        string[] pair = str.Split('=');
                        r = ProcessNameValuePair(pair, ref confValues);
                    }
                    else
                    {
                        string command = string.Empty;
                        string value = string.Empty;

                        str = str.Substring(0, str.Length - LINE_CONTINUATION.Length);
                        command = str.Substring(0, str.IndexOf('=') - 1).Trim();
                        value = str.Substring(str.IndexOf('=') + 1, str.Length - str.IndexOf('=') - 1).Trim();
                        if (value != string.Empty) value += ";";

                        string nextLine = string.Empty;
                        bool lineContinuation = false;

                        do
                        {
                            nextLine = sr.ReadLine();
                            lineContinuation = nextLine.LastIndexOf(LINE_CONTINUATION) == nextLine.Length - LINE_CONTINUATION.Length;
                            if (lineContinuation)
                            {
                                value += nextLine.Substring(0, nextLine.Length - LINE_CONTINUATION.Length - 1).Trim();
                                value += ";";
                            }
                        } while (lineContinuation);
                        value += nextLine.Trim();
                        r = ProcessNameValuePair(new string[] {command, value}, ref confValues);
                    }
                    if (r != string.Empty) sb.AppendLine(r);
                }
            }
            catch (Exception ex)
            {
                sb.AppendLine(ex.ToString());
            }

            err = sb.ToString();
        }

        private string ProcessNameValuePair(string[] pair, ref ConfigurationValues confValues)
        {
            string r = string.Empty;

            switch (pair[0].Trim().ToUpper())
            {
                case "ACTION":
                    confValues.Action = RemoveLineComment(pair[1].Trim());
                    break;
                case "SOURCE_DIR":
                    confValues.SrcDir = RemoveLineComment(pair[1].Trim());
                    break;
                case "DESTINATION_DIR":
                    confValues.DestDir = RemoveLineComment(pair[1].Trim());
                    break;
                case "INPUT":
                    confValues.InputFiles = RemoveLineComment(pair[1].Trim());
                    break;
                case "OUTPUT":
                    confValues.OutputFile = RemoveLineComment(pair[1].Trim());
                    break;
                default:
                    r = string.Format("*** ERROR ***: Key {0} not recognized.", pair[0]);
                    break;
            }

            return r;
        }

        private string RemoveLineComment(string s)
        {
            int idx = s.LastIndexOf(COMMENT_CHAR);

            if (idx > 0)
            {
                return s.Substring(0, idx - 1);
            }
            else
            {
                return s;
            }
        }
        #endregion
    }
}
