#region "Copyright"
/********************************************************************************
 * Copyright (c) 2007, Kerry D. Wong
 * http://www.kerrywong.com
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *   * Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *   * Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *   * Neither the name of Kerry D. Wong nor the
 *     names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY KERRY D. WONG "AS IS" AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL KERRY D. WONG BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *******************************************************************************/
#endregion

#region "using"
using System;
using System.IO;
using System.Collections;
using System.Collections.Generic;
using System.Text;

using KerryWong.TiffMS.Utils;
#endregion

namespace KerryWong.TiffMS
{
    public class Program
    {
        #region "private functions"
        private string CopyrightsStr()
        {
            StringBuilder sb = new StringBuilder();
            sb.AppendLine();
            sb.AppendLine("*******************************************************");
            sb.AppendLine("Tiff Merge/Split Utility.");
            sb.AppendLine("Copyright (C) Kerry D. Wong, 2007. All rights reserved.");
            sb.AppendLine("This sofware is copyrighted under BSD license.");
            sb.AppendLine("*******************************************************");
            return sb.ToString();
        }

        private string UsageStr()
        {
            StringBuilder sb = new StringBuilder();

            sb.AppendLine("Usage: TiffMS [-h|-s|-o -m] | [-c]");
            sb.AppendLine("\t-h show help");
            sb.AppendLine("\t-s split multi-page tiff file(s) into single-page tiff files");
            sb.AppendLine("\t-o output file name");
            sb.AppendLine("\t-m merge single-page tiff files into a single multi-page tiff file");
            sb.AppendLine("\t-c the configuration file name");
            sb.AppendLine();
            sb.AppendLine("Examples:");
            sb.AppendLine("\tTiffMS -s a.tiff");
            sb.AppendLine("\t\tsplit a.tiff into a_1.tiff a_2.tiff ...");
            sb.AppendLine("\tTiffMS -s a.tiff c:\\temp");
            sb.AppendLine("\t\tsplit a.tiff and put splitted files into c:\\temp.");
            sb.AppendLine("\tTiffMS -o out.tiff -m a.tiff b.tiff");
            sb.AppendLine("\t\tMerge a.tiff and b.tiff into a single tiff file out.tiff.");
            sb.AppendLine("\tTiffMS -c test.config");
            sb.AppendLine("\t\tMerge or split files according to settings in test.config");
            return sb.ToString();
        }

        private void ProcessCommandLine(string[] args)
        {
            MSUtils util = new MSUtils();

            string invalidSwitchMessage = string.Concat("*** ERROR ***: Invalid switch {0}!", Environment.NewLine);

            switch (args.Length)
            {
                case 0:
                    Console.WriteLine(CopyrightsStr());
                    Console.WriteLine(UsageStr());
                    break;
                case 1:
                    if (args[0].ToUpper().Trim() != "-H")
                    {
                        Console.WriteLine(string.Format(invalidSwitchMessage, args[0]));
                    } 

                    Console.WriteLine(UsageStr());
                    break;
                default:
                    string option = args[0].ToUpper().Trim();

                    if (option == "-S")
                    {
                        string destFolder = Environment.CurrentDirectory;

                        if (args.Length > 2)
                        {
                            destFolder = args[2];
                        }

                        util.ConvertToSinglePageTiffs(args[1].Trim(), destFolder);
                    }
                    else if (option == "-O")
                    {
                        if (args.Length < 4 || args[2].ToUpper().Trim() != "-M")
                        {
                            Console.WriteLine(string.Format(invalidSwitchMessage, "options"));
                        }
                        else
                        {
                            string destFile = args[1].Trim();
                            List<string> fileList = new List<string>();

                            for (int i = 3; i < args.Length; i++)
                            {
                                fileList.Add(args[i]);
                            }

                            util.ConvertToMultiPageTiff(fileList, destFile);
                        }
                    }
                    else if (option == "-C")
                    {
                        if (args.Length != 2)
                        {
                            Console.WriteLine(string.Format(invalidSwitchMessage, "options"));
                        }
                        else
                        {
                            if (!File.Exists(args[1]))
                            {
                                Console.WriteLine(String.Format("*** ERROR ***: Configuration File {0} does not exist.", args[1]));
                            }
                            else
                            {
                                Config cfg = new Config();
                                string err = string.Empty;
                                ConfigurationValues values = cfg.GetConfigurations(args[1],ref err);

                                if (err != string.Empty) Console.WriteLine(err);

                                string[] fns = values.InputFiles.Split(';');
                                switch (values.Action)
                                {                                        
                                    case Config.ACTION_SPLIT:                                        
                                        foreach (string fn in fns)
                                        {
                                            util.ConvertToSinglePageTiffs(Path.Combine(values.SrcDir,fn), Path.Combine(values.DestDir,Path.GetFileNameWithoutExtension(fn)));
                                        }
                                        break;
                                    case Config.ACTION_MERGE:
                                        fns = values.InputFiles.Split(';');
                                        List<string> list = new List<string>();
                                        foreach (string fn in fns)
                                        {
                                            Console.WriteLine(values.SrcDir);
                                            list.Add(Path.Combine(values.SrcDir, fn));                                     
                                        }
                                        util.ConvertToMultiPageTiff(list, Path.Combine(values.DestDir,values.OutputFile));
                                        break;
                                    default:
                                        Console.WriteLine(String.Format("*** ERROR ***: Unrecognized configuration key {0}.", values.Action));
                                        break;
                                }
                            }
                        }
                    }
                    break;
            }
        }
        #endregion

        #region "entry point"
        static void Main(string[] args)
        {
            Program p = new Program();
            p.ProcessCommandLine(args);
        }
        #endregion
    }
}